/* src/config.h.  Generated by configure.  */
/* src/config.h.in.  Generated from configure.ac by autoheader.  */

// Merges between win32 and linux version by Tue

#ifdef WIN32

#pragma warning(disable: 4305)

  /* Set to 1 if compiling for Win32 */
  #define OS_IS_WIN32 1

  /* Set to 1 if the compile is GNU GCC. */
  #define COMPILER_IS_GCC 0

#else

  /* Set to 1 if compiling for Win32 */
  #define OS_IS_WIN32 0

  /* Set to 1 if the compile is GNU GCC. */
  #define COMPILER_IS_GCC 1

  /* Major version of GCC or 3 otherwise. */
  #define GCC_MAJOR_VERSION 3

  /* Define to 1 if you have the <dlfcn.h> header file. */
  #define HAVE_DLFCN_H 1

  /* Define to 1 if you have the <inttypes.h> header file. */
  #define HAVE_INTTYPES_H 1

  /* Define to 1 if you have the `fftw' library (-lfftw). */
  #define HAVE_LIBFFTW 1

  /* Define to 1 if you have the `m' library (-lm). */
  #define HAVE_LIBM 1

  /* Define to 1 if you have the `rfftw' library (-lrfftw). */
  #define HAVE_LIBRFFTW 1

  /* Define if you have C99's lrint function. */
  #define HAVE_LRINT 1

  /* Define if you have C99's lrintf function. */
  #define HAVE_LRINTF 1

  /* Define to 1 if you have the <stdint.h> header file. */
  #define HAVE_STDINT_H 1

#endif



/* Set to 1 to enable debugging. */
#define ENABLE_DEBUG 0

/* Define to 1 if you have the `calloc' function. */
#define HAVE_CALLOC 1

/* Define to 1 if you have the `ceil' function. */
#define HAVE_CEIL 1

/* Define to 1 if you have the `floor' function. */
#define HAVE_FLOOR 1

/* Define to 1 if you have the `fmod' function. */
#define HAVE_FMOD 1

/* Define to 1 if you have the `free' function. */
#define HAVE_FREE 1

/* Define to 1 if you have the `efence' library (-lefence). */
/* #undef HAVE_LIBEFENCE */

/* Define to 1 if you have the `malloc' function. */
#define HAVE_MALLOC 1

/* Define to 1 if you have the `memcpy' function. */
#define HAVE_MEMCPY 1

/* Define to 1 if you have the `memmove' function. */
#define HAVE_MEMMOVE 1

/* Define to 1 if you have the <memory.h> header file. */
#define HAVE_MEMORY_H 1

/* Set to 1 if you have libsndfile. */
#define HAVE_SNDFILE 1

/* Define to 1 if you have the <stdlib.h> header file. */
#define HAVE_STDLIB_H 1

/* Define to 1 if you have the <strings.h> header file. */
#define HAVE_STRINGS_H 1

/* Define to 1 if you have the <string.h> header file. */
#define HAVE_STRING_H 1

/* Define to 1 if you have the <sys/stat.h> header file. */
#define HAVE_SYS_STAT_H 1

/* Define to 1 if you have the <sys/types.h> header file. */
#define HAVE_SYS_TYPES_H 1

/* Define to 1 if you have the <unistd.h> header file. */
#define HAVE_UNISTD_H 1

/* Name of package */
#define PACKAGE "libsamplerate"

/* Define to the address where bug reports for this package should be sent. */
#define PACKAGE_BUGREPORT ""

/* Define to the full name of this package. */
#define PACKAGE_NAME ""

/* Define to the full name and version of this package. */
#define PACKAGE_STRING ""

/* Define to the one symbol short name of this package. */
#define PACKAGE_TARNAME ""

/* Define to the version of this package. */
#define PACKAGE_VERSION ""

/* The size of a `double', as computed by sizeof. */
#define SIZEOF_DOUBLE 8

/* The size of a `float', as computed by sizeof. */
#define SIZEOF_FLOAT 4

/* The size of a `int', as computed by sizeof. */
#define SIZEOF_INT 4

/* The size of a `long', as computed by sizeof. */
#define SIZEOF_LONG 4

/* Define to 1 if you have the ANSI C header files. */
#define STDC_HEADERS 1

/* Version number of package */
#define VERSION "0.0.15"
