/**********************************************
 * cpuinfo.h - Case Metrics Interface
 *  Copyright 2007 John Sully.
 *
 *  This file is part of Case Metrics.
 *
 *  Case Metrics is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as 
 *  published by the Free Software Foundation.
 *
 *  Case Metrics is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Case Metrics.  If not, see <http://www.gnu.org/licenses/>.
 *
 **********************************************/


/*****************************************************
 * CPUINFO.H - cross platform cpuinfo struct contract
 *
 *****************************************************/
#include "../globaldefs.h"

#ifndef __CPUINFO__
#define __CPUINFO__


#define CPUID_VERSION 1

XCHAR *cpuInfoStz();

#endif //__CPUINFO__
