#ifndef __RECORDING_DEFS_H__
#define __RECORDING_DEFS_H__

#define RECORDING_PREF_KEY "[Recording]"
#define ENCODING_WAVE "WAV"
#define ENCODING_FLAC "FLAC"
#define ENCODING_AIFF "AIFF"
#define ENCODING_OGG  "OGG"
#define ENCODING_MP3 "MP3"

#define RECORD_READY 0.50f
#define RECORD_ON 1.0f
#define RECORD_OFF 0.0f

#endif
